/*
 ## Cypress FX3 Camera Kit source file (sensor.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2012,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file implements the I2C based driver for the MT9M114 image sensor used
   in the FX3 HD 720p camera kit.

   Please refer to the Aptina MT9M114 sensor datasheet for the details of the
   I2C commands used to configure the sensor.
 */

#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3dma.h>
#include <cyu3error.h>
#include <cyu3uart.h>
#include <cyu3i2c.h>
#include <cyu3types.h>
#include <cyu3gpio.h>
#include <cyu3utils.h>
#include "sensor.h"
#include "i2c_interface.h"

static void Step1_Post_Reset (void);
static void Step2_PLL_Timing (void);
static void Step3_Recommended (void);
static void Step4_APGA (void);
static void Step5_AWB_CCM (void);
static void Step7_PIPE_Preference (void);
static void Step8_Features (void);

/* This function inserts a delay between successful I2C transfers to prevent
   false errors due to the slave being busy.
 */
static I2COBJ_t s_SensorI2CObj;

int SensorI2CCfg(uint8_t slave_addr, uint8_t addr_len, uint8_t data_len)
{
	i2c_interface_init(&s_SensorI2CObj, slave_addr, addr_len, data_len);
	return 0;
}

int SensorWriteReg(uint32_t addr, uint32_t value)
{
	return (*s_SensorI2CObj.Write)(&s_SensorI2CObj, addr, value);
}

int SensorReadReg(uint32_t addr, uint32_t *p_value)
{
	return (*s_SensorI2CObj.Read)(&s_SensorI2CObj, addr, p_value);
}

int SensorInit(void)
{
	CyU3PDebugPrint (4, "SensorInit in  \r\n");
	//sensor_power_init();
	CyU3PThreadSleep (10);
	Sensor_PowerDn_Reset();
	CyU3PThreadSleep (10);
	SensorI2CCfg(0x70,1,1);
	SensorWriteReg(0x01,0x01);
	SensorScaling_HD720p_30fps();
	CyU3PDebugPrint (4, "SensorInit  out \r\n");
	return 0;
}



static void
SensorI2CAccessDelay (
        CyU3PReturnStatus_t status)
{
    /* Add a 10us delay if the I2C operation that preceded this call was successful. */
    if (status == CY_U3P_SUCCESS)
        CyU3PBusyWait (10);
}

/* Write to an I2C slave with two bytes of data. */
CyU3PReturnStatus_t
SensorWrite2B (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t highData,
        uint8_t lowData)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t  preamble;
    uint8_t buf[2];

    /* Validate the I2C slave address. */
   if ((slaveAddr != SENSOR_ADDR_WR) && (slaveAddr != I2C_MEMORY_ADDR_WR))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    /* Set the parameters for the I2C API access and then call the write API. */
    preamble.buffer[0] = slaveAddr;
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.length    = 3;             /*  Three byte preamble. */
    preamble.ctrlMask  = 0x0000;        /*  No additional start and stop bits. */

   buf[0] = highData;
    buf[1] = lowData;

    apiRetStatus = CyU3PI2cTransmitBytes (&preamble, buf, 2, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorWrite (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t count,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    /* Validate the I2C slave address. */
    if ((slaveAddr != SENSOR_ADDR_WR) && (slaveAddr != I2C_MEMORY_ADDR_WR))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    if (count > 64)
    {
        CyU3PDebugPrint (4, "ERROR: SensorWrite count > 64\n");
        return 1;
    }

    /* Set up the I2C control parameters and invoke the write API. */
    preamble.buffer[0] = slaveAddr;
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.length    = 3;
    preamble.ctrlMask  = 0x0000;

    apiRetStatus = CyU3PI2cTransmitBytes (&preamble, buf, count, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorRead2B (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    if ((slaveAddr != SENSOR_ADDR_RD) && (slaveAddr != I2C_MEMORY_ADDR_RD))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }

    preamble.buffer[0] = slaveAddr & I2C_SLAVEADDR_MASK;        /*  Mask out the transfer type bit. */
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.buffer[3] = slaveAddr;
    preamble.length    = 4;
    preamble.ctrlMask  = 0x0004;                                /*  Send start bit after third byte of preamble. */

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, buf, 2, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

CyU3PReturnStatus_t
SensorRead (
        uint8_t slaveAddr,
        uint8_t highAddr,
        uint8_t lowAddr,
        uint8_t count,
        uint8_t *buf)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;

    /* Validate the parameters. */
    if ((slaveAddr != SENSOR_ADDR_RD) && (slaveAddr != I2C_MEMORY_ADDR_RD))
    {
        CyU3PDebugPrint (4, "I2C Slave address is not valid!\n");
        return 1;
    }
    if ( count > 64 )
    {
        CyU3PDebugPrint (4, "ERROR: SensorWrite count > 64\n");
        return 1;
    }

    preamble.buffer[0] = slaveAddr & I2C_SLAVEADDR_MASK;        /*  Mask out the transfer type bit. */
    preamble.buffer[1] = highAddr;
    preamble.buffer[2] = lowAddr;
    preamble.buffer[3] = slaveAddr;
    preamble.length    = 4;
    preamble.ctrlMask  = 0x0004;                                /*  Send start bit after third byte of preamble. */

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, buf, count, 0);
    SensorI2CAccessDelay (apiRetStatus);

    return apiRetStatus;
}

/*
 * Reset the MT9M114 sensor using GPIO.
 */
void
Sensor_PowerDn_Reset (
        void)
{
    CyU3PReturnStatus_t apiRetStatus;

    /* Drive the GPIO high to powerdown sensor  */
    apiRetStatus = CyU3PGpioSetValue (SENSOR_POWERDN_GPIO, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "GPIO Set Value Error, Error Code = %d\n", apiRetStatus);
        return;
    }

    /* Delay the allow the sensor to power up. */
    CyU3PThreadSleep (10);

    /* Drive the GPIO high to reset the sensor.CyFalse  CyTrue*/
    apiRetStatus = CyU3PGpioSetValue (SENSOR_RESET_GPIO, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "GPIO Set Value Error, Error Code = %d\n", apiRetStatus);
        return;
    }

    /* Wait for some time to allow proper reset. */
    CyU3PThreadSleep (10);

    return;
}


/*
 * As UVC driver only support YUY2 uncompressed video, select YUY2 video format on the sensor.
 */
static void
SensorSelectYUY2Format (
        void)
{
    uint8_t buf[2];

    SensorRead2B (SENSOR_ADDR_RD, 0xC8, 0x6C, buf);
    buf[1] |= (1 << 1);
   // SensorWrite2B (SENSOR_ADDR_WR, 0xC8, 0x6C, buf[0], buf[1]);
}

/* MT9M114 sensor initialization sequence.
   Adapted from MT9M114-REV2.ini file provided by Aptina.

   Step1_Post_Reset       : Post-Reset Settings
   Step2_PLL_Timing       : PLL and Timing settings
   Step3_Recommended      : Patch, Errata and Sensor optimization Setting
   Step4_APGA             : APGA
   Step5_AWB_CCM          : AWB & CCM
   Step7_PIPE_Preference  : Color Pipe preference settings
   Step8_Features         : Ports, special features etc.
   Select Video Resolution
   SensorChangeConfig     : Update sensor configuration.
*/
//void
//SensorInit (
//        void)
//{
//   if (SensorI2cBusTest () != CY_U3P_SUCCESS)        /* Verify that the sensor is connected. */
//    {
//        CyU3PDebugPrint (4, "Error: Reading Sensor ID failed!\r\n");
//       return;
//    }

//    Step1_Post_Reset ();        /* Post-Reset Settings */
//    Step2_PLL_Timing ();        /* PLL and Timing */
//    Step3_Recommended ();       /* Patch, Errata and Sensor optimization Setting */
//    Step4_APGA ();              /* APGA */
//    Step5_AWB_CCM ();           /* AWB & CCM */
//    Step7_PIPE_Preference ();   /* Color Pipe preference settings, if any */
//    Step8_Features ();          /* Ports, special features, etc. */

    /* Update sensor configuration based on desired video stream parameters. Using 720p 30fps as default setting.*/
//    SensorScaling_HD720p_30fps ();

//   SensorSelectYUY2Format ();
//    SensorChangeConfig ();
//}

/*
 * Verify that the sensor can be accessed over the I2C bus from FX3.
 */
uint8_t
SensorI2cBusTest (
        void)
{
    /* The sensor ID register can be read here to verify sensor connectivity. */
    uint8_t buf[2];

    /* Reading sensor ID */
    if (SensorRead2B (SENSOR_ADDR_RD, 0x00, 0x00, buf) == CY_U3P_SUCCESS)
    {
        if ((buf[0] == 0x24) && (buf[1] == 0x81))
        {
            return CY_U3P_SUCCESS;
        }
    }
    return 1;
}

/* Post sensor reset settings. */
static void
Step1_Post_Reset (
        void)
{
    uint8_t buf[2];

    /* Poll the command register for ready status. */
    while (1)
    {
        if (SensorRead2B (SENSOR_ADDR_RD, 0x00, 0x80, buf) != CY_U3P_SUCCESS)
            CyU3PDebugPrint (4, "Error: Command register read failed\r\n");
        else
            if (!(buf[1] & 0x02))
                break;
    }

    /* Errata item 2 */
    SensorRead2B (SENSOR_ADDR_RD, 0x30, 0x1A, buf);
    buf[0] |= (1 << 2);
  //  SensorWrite2B (SENSOR_ADDR_WR, 0x30, 0x1A, buf[0], buf[1]);
}

/* Update sensor PLL and timing settings.
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
static void
Step2_PLL_Timing (
        void)
{


}

/* Patch, errata and optimization settings from Aptina.
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
static void
Step3_Recommended (
        void)
{

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
static void
Step4_APGA (
        void)
{

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
static void
Step5_AWB_CCM (
        void)
{
    /* CCM */

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
static void
Step7_PIPE_Preference (
        void)
{

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
static void
Step8_Features (
        void)
{
    /* REG = 0x098E, 0x0000 : Set XDMA to logical addressing */
    /* REG = 0xC984, 0x8040 : cam_port_output_control = 32832 */
    /* REG = 0x001E, 0x0777 : PAD SLEW CONTROL */
    /* IF_SERIAL=0xCA, 0x08, 0xFF, 8:16, ==0x10, LOAD=MIPI settings */

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
void
SensorScaling_VGA (
        void)
{

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
void
SensorScaling_HD720p_30fps (
        void)
{

}

/*
   The procedure is adapted from Aptina's sensor initialization scripts. Please
   refer to the MT9M114 sensor datasheet for details.
 */
void
SensorChangeConfig (
        void)
{
    uint8_t buf[2];

    /* 1. set next state is change_config */
    buf[0] = 0x28;  /* sys_state_enter_config_change */
    SensorWrite (SENSOR_ADDR_WR, 0xDC, 0x00, 1, buf);

    /* 2. poll command register */
    while (SensorRead2B(SENSOR_ADDR_RD, 0x00, 0x80, buf))
    {
        if (!(buf[1]& 0x02))
            break;
    }

    /* 3. issue command */
  //  SensorWrite2B (SENSOR_ADDR_WR, 0x00, 0x80, 0x80, 0x02);

    /* 4. waiting for firmware updating new configuration */
    while (SensorRead2B(SENSOR_ADDR_RD, 0x00, 0x80, buf))
    {
        if (!(buf[1]& 0x02))
            break;
    }

    /* 5. error handling */
    SensorRead2B (SENSOR_ADDR_RD, 0x00, 0x80, buf);

    /* 6. refresh command */
   // SensorWrite2B (SENSOR_ADDR_WR, 0x00, 0x80, 0x80, 0x04);
    while (SensorRead2B(SENSOR_ADDR_RD, 0x00, 0x80, buf))
    {
        if (!(buf[1]& 0x04))
            break;
    }

    SensorRead2B (SENSOR_ADDR_RD, 0xDC, 0x01, buf);
}

/*
   Get the current brightness setting from the MT9M114 sensor.
 */
uint8_t
SensorGetBrightness (
        void)
{
    uint8_t buf[2];

    SensorRead2B (SENSOR_ADDR_RD, 0xCC, 0x0A, buf);
    return (uint8_t)buf[1];
}

/* TODO #2-2 sensor function to read gain register */
/* Copy the SensorGetBrightness function (right aobve this comment),
 * paste it below this comment and rename it to SensorGetGain.
 * Change the register address accessed to 0xCC12.
 */

/*
   Update the brightness setting for the MT9M114 sensor.
 */
void
SensorSetBrightness (
        uint8_t input)
{
    //SensorWrite2B (SENSOR_ADDR_WR, 0xCC, 0x0A, 0x00, input);
}

/* TODO #2-3 sensor function to write gain register */
/* Copy the SensorSetBrightness function (right above),
 * paste it below this comment and rename it to SensorSetGain.
 * Change the register address accessed to 0xCC12.
 */


